@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2021 Tobias Stoeckmann <tobias@stoeckmann.org>
+ * Copyright (c) 2025 Tobias Stoeckmann <tobias@stoeckmann.org>
  *
  * Permission to use, copy, modify, and distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -495,7 +495,7 @@ process_atoms(xcb_connection_t *c, xcb_screen_t *screen, xcb_pixmap_t *pixmap,
 {
 	static xcb_void_cookie_t (*delete)(xcb_connection_t *, uint32_t) =
 	    xcb_kill_client;
-	int deleted, i;
+	int i;
 	xcb_intern_atom_cookie_t atom_cookie[2];
 	xcb_intern_atom_reply_t *atom_reply[2];
 	xcb_get_property_cookie_t property_cookie[2];
@@ -529,17 +529,16 @@ process_atoms(xcb_connection_t *c, xcb_screen_t *screen, xcb_pixmap_t *pixmap,
 			old[i] = NULL;
 	}
 
-	deleted = 0;
 	if (old[0] != NULL && pixmap != NULL && *old[0] != *pixmap) {
 		delete(c, *old[0]);
-		deleted = 1;
 	}
 	if (old[1] != NULL && (old[0] == NULL || *old[0] != *old[1])) {
 		delete(c, *old[1]);
-		deleted = 1;
 	}
-	if (deleted)
+	if (pixmap != NULL) {
+		/* kill clients only once, otherwise we might kill ourself */
 		delete = xcb_free_pixmap;
+	}
 
 	if (old_pixmap != NULL) {
 		if (old[0] != NULL && old[1] != NULL && *old[0] == *old[1])
@@ -593,11 +592,13 @@ process_screen(xcb_connection_t *c, xcb_screen_t *screen, int snum,
 		/* fake an output that fits the picture */
 		width = pixman_image_get_width(pixman_image);
 		height = pixman_image_get_height(pixman_image);
-		tile_output.x = 0;
-		tile_output.y = 0;
-		tile_output.width = width;
-		tile_output.height = height;
-		tile_output.name = NULL;
+		tile_output = (wp_output_t){
+			.x = 0,
+			.y = 0,
+			.width = width,
+			.height = height,
+			.name = NULL
+		};
 		outputs = &tile_output;
 	} else {
 		width = screen->width_in_pixels;
@@ -631,10 +632,12 @@ process_screen(xcb_connection_t *c, xcb_screen_t *screen, int snum,
 		    width, height);
 		gc = xcb_generate_id(c);
 		xcb_create_gc(c, gc, pixmap, 0, NULL);
-		rectangle.x = 0;
-		rectangle.y = 0;
-		rectangle.width = width;
-		rectangle.height = height;
+		rectangle = (xcb_rectangle_t){
+			.x = 0,
+			.y = 0,
+			.width = width,
+			.height = height
+		};
 		xcb_poly_fill_rectangle(c, pixmap, gc, 1, &rectangle);
 		created = 1;
 	} else {
